{-----------------------------------------------------------------------------
 Unit Name: SynthInstruments
 Author:    Sebastian Htter
 Date:      2006-10-29
 Purpose:   Wave functions for instruments
            Parts taken from a Tutorial by Nico Michaelis

 History:   2006-10-29 initial release
 
------------------------------------------------------------------------------
 Copyright Notice: If you copy or modify this file, this block
   _must_ be preserved! If you are using this code in your projects,
   I would like to see a word of thanks in the About-Box or a similar place.
-----------------------------------------------------------------------------}
unit SynthInstruments;

interface

uses SynthUtils;

function PlayPiano(X,Freq,Duration:Single): Single;
function PlayClavinetA(X,Freq,Duration:Single): Single;
function PlayClavinetB(X,Freq,Duration:Single): Single;
function PlayEOrgan(X,Freq,Duration:Single): Single;

function PlayFlute(X,Freq,Duration:Single): Single;
function PlayTrumpet(X,Freq,Duration:Single): Single;

function PlayBell(X,Freq,Duration:Single): Single;

function PlayDrum(X,Freq,Duration:Single): Single;
function PlaySynthDrum(X,Freq,Duration:Single): Single;
function PlaySnare(X,Freq,Duration:Single): Single;
function PlayHat(X,Freq,Duration:Single): Single;

const
  //Hllkurveneinstellungen

  (* Klavier *)
  Klavier_Waves_Attack   = 4;
  Klavier_Waves_MaxDecay = 9600;
  Klavier_Waves_End = 500;

  (* Elektro-Orgel *)
  EOrgel_Waves_Attack = 20;
  EOrgel_Waves_Decay = 80;
  EOrgel_Waves_End = 150;

  (* Flte *)
  Floete_Waves_Attack = 50;
  Floete_Waves_End    = 20;

  (* Glocke *)
  Glocke_Waves_Attack   = 15;
  Glocke_Waves_MaxDecay = 40000;
  Glocke_Waves_End = 1800;

  Drum_Waves_Attack   = 1;
  Drum_Waves_Sustain  = 6;
  Drum_Waves_Release  = 2;

  Snare_Waves_Sustain = 3;
  Snare_Waves_Release = 25;

  Hat_Waves_Attack   = 0;
  Hat_Waves_Release  = 20;

implementation

function PlayPiano(X,Freq,Duration:Single): Single;
{ Erzeugt normierte Samples fr Klaviernoten:
  Obertne, Schwebungen }
begin
  //Sinuskurve
  Result :=          Harmonics(x, Freq);
  Result := Result + Harmonics(x, Freq * 1.0015);
  Result := Result + Harmonics(x, Freq * 1.0025);
  Result := 1/3.0 * Result;
  //Envelope
  Result := Result * ADEnvelopeCutOff(x, Klavier_Waves_Attack/Freq,
              Duration - Klavier_Waves_Attack/Freq -
              Klavier_Waves_End/Freq, Klavier_Waves_MaxDecay / Freq,
              Klavier_Waves_End/Freq, 1.0, 0.0);
end;

function PlayClavinetA(X,Freq,Duration:Single): Single;
{ Erzeugt normierte Samples fr Klavinettnoten:
  Obertne, Schwebungen. Ein Klavinett ist einem Klavier sehr
  hnlich, nur klingt es ein wenig "hrter" }
begin
  //Sinuskurve
  Result :=          TriangleHarmonics(x, Freq);
  Result := Result + TriangleHarmonics(x, Freq * 1.0015);
  Result := Result + TriangleHarmonics(x, Freq * 1.0025);
  Result := 1/3.0 * Result;
  //Envelope
  Result := Result * ADEnvelopeCutOff(x, Klavier_Waves_Attack/Freq,
              Duration - Klavier_Waves_Attack/Freq -
              Klavier_Waves_End/Freq, Klavier_Waves_MaxDecay / Freq,
              Klavier_Waves_End/Freq, 1.0, 0.0);
end;

function PlayClavinetB(X,Freq,Duration:Single): Single;
{ Erzeugt normierte Samples fr Klavinettnoten:
  Obertne, Schwebungen. Ein Klavinett ist einem Klavier sehr
  hnlich, nur klingt es ein wenig "hrter" }
begin
  //Sinuskurve
  Result :=          SawtoothHarmonics(x, Freq);
  Result := Result + SawtoothHarmonics(x, Freq * 1.0015);
  Result := Result + SawtoothHarmonics(x, Freq * 1.0025);
  Result := 1/3.0 * Result;
  //Envelope
  Result := Result * ADEnvelopeCutOff(x, Klavier_Waves_Attack/Freq,
              Duration - Klavier_Waves_Attack/Freq -
              Klavier_Waves_End/Freq, Klavier_Waves_MaxDecay / Freq,
              Klavier_Waves_End/Freq, 1.0, 0.0);
end;

function PlayEOrgan(X,Freq,Duration:Single): Single;
{ Erzeugt normierte Samples fr Elektro-Orgeln, die
  hufig bei Gospeln zum Einsatz kommen. Die schwebung ist
  sehr ungewhnlich und klingt sehr charakteristisch. Die
  Hllkurve verluft ein wenig anders, als beim Klavier. }
begin
  //Sinuskurve
  Result :=          Harmonics(x, Freq);
  Result := Result + Harmonics(x, Freq * 1.0175);
  Result := Result + Harmonics(x, Freq * 1.025);
  Result := 1/3.0 * Result;
  //Envelope
  Result := Result * ADSREnvelope(x, EOrgel_Waves_Attack/Freq,
                       EOrgel_Waves_Decay/Freq,
                       Duration - EOrgel_Waves_Attack/Freq -
                         EOrgel_Waves_Decay/Freq - EOrgel_Waves_End/Freq,
                       EOrgel_Waves_End/Freq, 1.0, 0.7);
end;

function PlayFlute(X,Freq,Duration:Single): Single;
{ Erzeugt normierte Samples fr Floeten/Melodikas.
  Das ganze luft ber sehr einfache Dreiecke. Eine Reihe
  von Verbesserungen wren sicher mglich. }
begin
  Result := WaveTriangle(x * Freq)*Sin(x * 2*Pi*Freq*2);
  //Envelope
  Result := Result * ASREnvelope(x, Floete_Waves_Attack/Freq,
                       Duration - Floete_Waves_Attack/Freq -
                         Floete_Waves_End/Freq,
                         Floete_Waves_End/Freq, 1.0);
end;

function PlayTrumpet(X,Freq,Duration:Single): Single;
{ Erzeugt normierte Samples fr Floeten/Melodikas.
  Das ganze luft ber sehr einfache Dreiecke. Eine Reihe
  von Verbesserungen wren sicher mglich. }
begin
  Result := WaveSawtooth(x * Freq)*Sin(x * 2*Pi*Freq*2);
  //Envelope
  Result := Result * ASREnvelope(x, Floete_Waves_Attack/Freq,
                       Duration - Floete_Waves_Attack/Freq -
                         Floete_Waves_End/Freq,
                         Floete_Waves_End/Freq, 1.0);
end;

function PlayBell(X,Freq,Duration:Single): Single;
{ Erzeugt normierte Samples fr Glocken: Untertne mit
  Schwebungen }
begin
  Result :=          SubHarminics(x, Freq);
  Result := Result + SubHarminics(x, Freq * 1.005);
  Result := Result + SubHarminics(x, Freq * 1.0075);
  Result := 1/3.0 * Result;

  //Envelope
  Result := Result * ADEnvelopeCutOff(x, Glocke_Waves_Attack/Freq,
              Duration - Glocke_Waves_Attack/Freq -
              Glocke_Waves_End/Freq, Glocke_Waves_MaxDecay / Freq,
              Glocke_Waves_End/Freq, 1.0, 0.0);
end;

function PlayDrum(X,Freq,Duration:Single): Single;
var p:single;
begin
  p:= X / Duration;
  if p < 1/3 then
    Freq:= (1-p*3) * 2 * Freq + Freq;
  Result:= WaveTriangle(X*Freq+0.5)*WaveSine(X * Freq);
  if X < 2/Freq then begin
    p:= 1 - 0.5 * (X/(2/Freq));
    Result := Result + Random * p - p / 2;
  end;

  //Envelope
  Result := Result * ASREnvelope(x, Drum_Waves_Attack/Freq,
              Drum_Waves_Sustain / Freq,
              Drum_Waves_Release / Freq, 1.0);
end;

function PlaySynthDrum(X,Freq,Duration:Single): Single;
var p:single;
begin
  p:= X / Duration;
  if p < 1/3 then
    Freq:= (1-p*3) * 2 * Freq + Freq;
  Result := WaveSine(X * Freq);

  //Envelope
  Result := Result * ASREnvelope(x, Drum_Waves_Attack/Freq,
              Drum_Waves_Sustain / Freq,
              Drum_Waves_Release / Freq, 1.7);
  if Result > 1.0 then
    Result:= 1.0;
  if Result < -1.0 then
    Result:= -1.0;     
end;

function PlaySnare(X,Freq,Duration:Single): Single;
var p:single;
begin
  Freq:= 140;
  Result := WaveSine(X * Freq);

  p:= 1 - 0.5 * (X/(2/Freq));
  Result := Result + Random * p - p / 2;

  //Envelope
  Result := Result * ASREnvelope(x, 0,
              Snare_Waves_Sustain / Freq,
              Snare_Waves_Release / Freq, 1.0);
end;

function PlayHat(X,Freq,Duration:Single): Single;
{ Erzeugt normierte Samples fr Glocken: Untertne mit
  Schwebungen }
begin
  Result := Random*2-1.0 + abs(sin(0.3*X*Freq));

  //Envelope
  Result := Result * ASREnvelope(x, Hat_Waves_Attack / Freq,
              0,
              Hat_Waves_Release / Freq, 1.0);
end;

end.
